/*
 * Sixth Data. Author: Svjatoslav Agejenko. 
 * This project is released under Creative Commons Zero (CC0) license.
 *
*/

package eu.svjatoslav.sixth.data.store.file;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class EntryAllocationTable {

    private final FileDataStore fileDataStore;

    public EntryAllocationTable(final FileDataStore fileDataStore) {
        this.fileDataStore = fileDataStore;
    }

    public void enlarge(final int newSize) throws IOException {
        final int oldSize = fileDataStore.metaData.getEntriesTableSize();

        for (int i = oldSize; i < newSize; i++) {
            final EntryRecord entryRecord = new EntryRecord(i, 0, 0);
            entryRecord.save(fileDataStore);
        }

        fileDataStore.metaData.setEntriesTableSize(newSize);
    }

    public long getEntryRecordLocation(final int entryId) {
        return (entryId * EntryRecord.ENTRY_RECORD_LENGTH)
                + MetaData.FILE_LOCATION_ENTRY_ALLOCATION_TABLE_START;
    }

    public int getNewUnusedEntryId() throws IOException {
        while (true) {

            final int newEntryId = fileDataStore.metaData.getNewEntryId();

            final EntryRecord entryRecord = new EntryRecord(fileDataStore,
                    newEntryId);

            if (!entryRecord.isUsed())
                return newEntryId;
        }
    }

    public void initializeNewFile() throws IOException {
        for (int i = 0; i < fileDataStore.metaData.getEntriesTableSize(); i++) {
            final EntryRecord entryRecord = new EntryRecord(i, 0, 0);
            entryRecord.save(fileDataStore);
        }
    }

    /**
     * Sorted list of @link {@link EntryRecord}'s.
     */
    public List<EntryRecord> loadAllEntryRecords() throws IOException {
        final List<EntryRecord> entryRecords = new ArrayList<>();

        for (int i = 0; i < fileDataStore.metaData.getEntriesTableSize(); i++) {
            final EntryRecord entryRecord = new EntryRecord(fileDataStore, i);
            if (entryRecord.isUsed())
                entryRecords.add(entryRecord);
        }

        Collections.sort(entryRecords);

        return entryRecords;
    }

}
